/********************************************************************
  *
  *  program     : HTMLTemplate.java
  *  author      : Helge Janicke
  *  version     : 2000-12-24(001)
  *  (c)         : Helge Janicke
  *  email       : heljanic@agent.fho-emden.de
  *  description : Replaces variable definitions in a HTML template
  *          file with the values that are specified in a
  *          Hashtable.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-11-30    heljanic    modified thrown Exeptions in
  *                           buildTemplate. See documentation.
  * -----------------------------------------------------------------
  * 2000-12-24    heljanic    adding a hashtable for default
  *                           variables.
  * -----------------------------------------------------------------
  * 2000-12-24    heljanic    adding features to load localized
  *                           templates.
  *                           loadFile(String, Locale);
  *                           loadTemplate(String, Locale);
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
********************************************************************/

package de.janicke.hj.tools.servlet;

import java.util.Hashtable;
import java.util.Locale;
import java.io.*;

/** HTMLTemplate is a class to replace parts in a HTML file with the
  * value variables. The parts to be replaced are marked up as the
  * following:<br>
  * <tt>&lt;!--[ VARNAME ]--></tt><br>
  * <tt>"&lt;!--["</tt> and <tt>"]-->"</tt> are the delimiters of a variables name. These
  * delimiters are specified by the constants BEGIN_VARIABLE and
  * END_VARIABLE and must be used in the HTML template files. The
  * substring in between is stripped of any leading and trailing spaces
  * and then treated as the name of a variable. The value of the variable
  * is retrieved out of a Hashtable (an implementation of hashtable) using
  * the variable name as key.
  * The recommended way to use this class is to generate a instance using
  * the standard constructor and then put variables into the Hashtable using
  * the <tt>put(String name, Object value)</tt>. Values can be retrieved
  * with the <tt>get(Object name)</tt> method.
  * To build up a HTML template file use the
  * <tt>buildTemplate(String htmlfile)</tt> method that actually does the
  * replacement.<br>
  * To spare time loading files it is also possible to load files into a hash
  * table using the <tt>loadTemplate</tt> methods. Which way to choose mainly depends on
  * the needed resources and memory available for this servlet. (Remember to set the
  * VM parameters if you need more memory for the servlet!)<br>
  * This Class supports a kind of localization. The loadFile and loadTemplate methods are
  * also provided with an extra Locale object.
  * If you plan to use localization please add for each language to support a subdirectory
  * in your template directory named as the provided Locale: e.g. "en_EN". The "localized" methods
  * then will look up the file in this directory instead of your default template directory.
  * If localization is not supported (the localized template file doesn't exist) the default file
  * will be taken.
  */
public class HTMLTemplate {

  /** Constant to indicate the beginning of a variable name in the
    * HTML string.
    */
  final static String BEGIN_VARIABLE = "<!--[";

  /** Constant to indicate the end of a variable name in the
    * HTML string.
    */
  final static String END_VARIABLE = "]-->";

  /** Hashtable to hold the variable names and values. In this
    * Hashtable the variable names are looked up when builing a
    * Templatefile.
    */
  private Hashtable variables;

  /** Hashtable to hold the defaults for variable names and values.
    * This hashtable is used to look u variables that are requested
    * by a template file and not provided in the variables hashtable.
    */
  private Hashtable defaultVariables;

  /** Hashtable to hold the HTML template files. */
  private Hashtable templates = new Hashtable();

  /** Constructs a new HTMLTemplate with an empty Hashtable for
    * the variables.
    */
  public HTMLTemplate() {
    setVariables(new Hashtable());
    setDefaultVariables(new Hashtable());
  }

  /** Constructs a new HTMLTemplate with the specified Hashtable for
    * the variables.
    * @param variables The Hashtable used for the variables.
    */
  public HTMLTemplate(Hashtable variables) {
    setVariables(variables);
    setDefaultVariables(new Hashtable());
  }

  /** Returns the Hashtable used for the variables.
    * @return The Hashtable for the variables.
    */
  public Hashtable getVariables() {
    return this.variables;
  }

  /** Sets the Hashtable for the variables.
    * @param the new Hashtable for the variables.
    */
  public void setVariables(Hashtable variables) {
    this.variables = variables;
  }

  /** Returns the Hashtable used for the defaultVariables.
    * @return The Hashtable for the defaultVariables.
    */
  public Hashtable getDefaultVariables() {
    return this.defaultVariables;
  }

  /** Sets the Hashtable for the defaultVariables.
    * @param the new Hashtable for the defaultVariables.
    */
  public void setDefaultVariables(Hashtable defaultVariables) {
    this.defaultVariables = defaultVariables;
  }


  /** Removes all variables, defaults and templates from the hash. */
  public void clear() {
    variables.clear();
    defaultVariables.clear();
    templates.clear();
  }

  /** Removes all variabels from the hash. */
  public void clearVariables() {
    variables.clear();
  }

  /** Removes all defaults from the hash. */
  public void clearDefaultVariables() {
    defaultVariables.clear();
  }

  /** Removes all templates from the hash. */
  public void clearTemplates() {
    templates.clear();
  }

  /** Puts the default for a variable into the Hashtable.
    * If the supplied name is not a String the Object.toString()
    * method is used to find the corresponding variable name in the
    * HTML - file.
    * @param name the name of the variable (which is used in the
    *     HTML - file.) This String will be stripped of leading and
    *     trailing spaces using the String.trim() method.
    * @param value the value of the variable.
    * @return The Object that was the previous default value of this
    *     variable.
    */
  public Object putDefault(String name, Object value) {
   return defaultVariables.put(name.trim(), value);
  }

  /** Returns the default value of a variable.
    * @param name The name of the variable.
    * @return The defualt value of the variable, null if there is no entry
    *         for this variable.
    */
  public Object getDefault(Object name) {
    return defaultVariables.get(name);
  }

  /** Puts a variable into the Hashtable.
    * If the supplied name is not a String the Object.toString()
    * method is used to find the corresponding variable name in the
    * HTML - file.
    * @param name the name of the variable (which is used in the
    *     HTML - file.) This String will be stripped of leading and
    *     trailing spaces using the String.trim() method.
    * @param value the value of the variable.
    * @return The Object that was the previous value of this
    *     variable.
    */
  public Object put(String name, Object value) {
    return variables.put(name.trim(), value);
  }

  /** Returns the value of a variable.
    * @param name The name of the variable.
    * @return The value of the variable, null if there is no entry
    *         for this variable.
    */
  public Object get(Object name) {
    return variables.get(name);
  }

  /** Puts a HTML template into the template hashtable.
    * @param key The key for the template file.
    * @param template The template file.
    * @return The template file that was previously associated with that key.
    */
  public Object putTemplate(Object key, Object template) {
    return templates.put( key, template );
  }

  /** Returns the HTML template that is associated with the key.
    * @param key The key for the template file.
    * @return The template file.
    */
  public Object getTemplate(Object key) {
    return templates.get(key);
  }

  /** Builds the template file with the specified key.
    * Get the template file out of the hash and calls buildTemplate(...).
    * @param key The key of the template file.
    * @return The build template file. <tt> null </tt> if the key wasn't found.
    * @throw NullPointerException, java.text.ParseException
    */
  public String makeTemplate(Object key) throws NullPointerException, java.text.ParseException {
    Object temp = getTemplate(key);
    String returnValue = "uncaught ERROR";
    if (temp != null) {
      try {
		System.err.println("makeTemplate calling buildtemplate (param = " + temp.toString());
        returnValue = buildTemplate(temp.toString());
      } catch ( NullPointerException exc) {
        throw new NullPointerException ( exc.getMessage() + key );
      }
      return returnValue;
    }
    return null;
  }

  /** Loads a template file from the specified location and puts it into the Hash.
    * The key is the <tt>template</tt> parameter.
    * @param template The path of the template file.
    * @return The key the file has in the Hash.
    * @throw IOException If the file couldn't be loaded.
    */
  public String loadTemplate(String template) throws IOException {
    putTemplate(template, loadFile(template) );
    return template;
  }

  /** Loads a template file from the specified location and puts it into the Hash.
    * @param template The path of the template file.
    * @param locale The local to be used to laod the file
    * @return The key the file has in the Hash.
    * @throw IOException If the file couldn't be loaded.
    * @see loadFile(String path, Locale locale)
    */
  public String loadTemplate(String template, Locale locale) throws IOException {
    putTemplate(template, loadFile(template, locale) );
    return template;
  }


  /** Loads a File from the specified path.
    * @param path The path of the file to open.
    * @return The String holding the contents of the file.
    * @throw IOException If the file couldn't be read.
    */
  public String loadFile(String path) throws IOException, FileNotFoundException {
    File f = new File(path);
    if (!f.exists()) throw new FileNotFoundException("Couldn't load template file: " + path );

    BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(path)));
    String input;
    StringBuffer file = new StringBuffer();

    input = in.readLine();

    while (input != null) {
      file.append(input);
      input = in.readLine();
    }

    return file.toString();
  }

  /** Loads a File from the specified path using a Locale.
    * This method tries to find a subdirectory with the locale identity e.g. "de_DE".
    * Actually the Locale.toString() method is used to look up the directory.
    * If the subdirectory exist it loads the file in the specified locale directory.
    * if the subdirectory is not found or the specified file doesn't exits in this
    * directory it tries to open the default file specified by <tt>path</tt>.
    * @param path The path of the file to open.
    * @param locale The locale identifying the directory to load the template from.
    * @return The String holding the contents of the file.
    * @throw IOException If the file couldn't be read.
    */
  public String loadFile(String path, Locale locale) throws IOException, FileNotFoundException {
    File loc = null;
    File in = new File(path);
    File parent = in.getParentFile();
    File tmp = new File(parent, locale.toString());
    loc = new File(tmp, in.getName());

    if (!loc.exists()) loc = in;

    return loadFile(loc.getAbsolutePath());
  }

  /** Searches the htmlFile for the START_VARIABLE and ENDVARIABLE
    * statements. The token in between is interpreted as the
    * variable name. If such a token is found, it is replaced by the
    * value of the variable.
    * if there is a variable encountered in the HTML file with no
    * corresponding variable in the Hashtable, then a Exception is
    * thrown.
    * @param htmlFile The HTML template file.
    * @return The html file with replaced values.
    * @throw NullPointerException if a requested variable name was not
    *     not found in the Hashtable.
    * @throw java.text.ParseException if a variable BEGIN_VARIABLE tag
    *     wasn't closed by a END_VARIABLE tag.
    */
  public String buildTemplate(String htmlFile) throws NullPointerException, java.text.ParseException {
    StringBuffer out = new StringBuffer(htmlFile);
    int beginIndex=-1;
    int beginVariable=0;
    int endIndex=0;
    int endVariable=0;
    boolean performCheck = true;
    String varName = "";
    Object replaceValue = null;

    if (htmlFile == null) return null;

    while (performCheck == true) {

        varName="";
      // search for first index of BEGIN_VARIABLE string constant.
        beginIndex = out.toString().indexOf(BEGIN_VARIABLE, beginIndex);
      if (beginIndex < 0) {
        // no BEGIN_VARIABLE string constant found.
          performCheck = false;
      } else {
        // BEGIN_VARIABLE string constant found at beginIndex.

        // Compute begin index of the Variablename.
        beginVariable = beginIndex + BEGIN_VARIABLE.length();

        // search for first index of END_VARIABLE string constant.
        endVariable = out.toString().indexOf(END_VARIABLE, beginVariable);
        if (endVariable < 0) {
          // there was a begin of a variable, but no end.
          throw new java.text.ParseException("Encountered a not TERMINATED VARIABLE at char " + beginVariable, beginVariable );
        } else {
          // ENDVARIABLE string constant found.

          // Compute end index in the HTML file.
          endIndex = endVariable + END_VARIABLE.length();

          // Retrieve the variable name.
          varName = out.substring(beginVariable, endVariable);

          // remove leading and trailing spaces.
          varName = varName.trim();

          // Retrieve the value of the variable out of the Hashtable.
          replaceValue = get(varName);
          if (replaceValue == null) replaceValue = getDefault(varName);

          if (replaceValue == null) {
            // found a variable request in HTML, but there is no entry in the Hashtable.
            throw new NullPointerException("No variable entry found for name = " + varName + "<br>found at beginVariable = " + beginVariable + "<br>in file: ");
          } else {
            // found variable in Hashtable.

            // Replace the whole HTML entry with the value of the variable.
            out.replace(beginIndex, endIndex, replaceValue.toString());
          }
        }

        beginIndex+=varName.length();
      }
    }

    return out.toString();
  }
}