/********************************************************************
  *
  *  program     : SimpleVocAdmin.java
  *  author      : Karsten Wolke, Niels-Peter de Witt
  *  version     : 2000-11-28(001)
  *  (c)         : Niels-Peter de Witt
  *  email       : ndewitt@gmx.de
  *  description : Handles the vocable administration with a database.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * -----------------------------------------------------------------
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
********************************************************************/

package de.janicke.hj.tools.servlet;

import java.sql.*;


/** Handles the vocable administration with a Database.
  */
public class SimpleVocAdmin extends AbstractVocAdmin implements VocAdmin {

  //-------------------------------------------------- PreparedStatements

    private Database database = null;

    /** Precompiled SQL statement to add a vocable by UserId. */
    private PreparedStatement preSQLAddVoc;

    /** Precompiled SQL statement to del a vocable by VocId and UserId. */
    private PreparedStatement preSQLDelVoc;

    /** Precompiled SQL statement to edit a vocable by VocId and UserId. */
    private PreparedStatement preSQLEditVoc;

    /** Precompiled SQL statement to edit the state of a vocable by VocId. */
    private PreparedStatement preSQLEditVocState;

    /** Precompiled SQL statement to add a Unit by UserId. */
    private PreparedStatement preSQLAddUnit;

    /** Precompiled SQL statement to del a Unit by UnitId and UserId. */
    private PreparedStatement preSQLDelUnit;

    /** Precompiled SQL statement to add a Language by UserId. */
    private PreparedStatement preSQLAddLanguage;

    /** Precompiled SQL statement to del a Language by LanguageId and UserId. */
    private PreparedStatement preSQLDelLanguage;

    /** Precompiled SQL statement to get a vocable-set by userId and language. */
    private PreparedStatement preSQLGetVocByLangId;

    /** Precompiled SQL statement to get a vocable-set
      * by userId and language and date.
      */
    private PreparedStatement preSQLGetVocByDate;

    /** Precompiled SQL statement to get a vocablet by vocId. */
    private PreparedStatement preSQLGetVocByVocId;

    /** Precompiled SQL statement to get a languagename-set of all languages. */
    private PreparedStatement preSQLGetAllLanguages;

    /** Precompiled SQL statement to get a unitnames-set of all languages. */
    private PreparedStatement preSQLGetAllUnits;

    /** Precompiled SQL statement to get a langId by languagename. */
    private PreparedStatement preSQLGetLangIdByName;

    /** Precompiled SQL statement to get a unitId by unitname.  */
    private PreparedStatement preSQLGetUnitIdByName;

  // ------------------------------------------------



  /** Constructor sets the Database.
    * @param database The used Database.
    */
  public SimpleVocAdmin(Database database) {
    this.database = database;
  }

// ----------------------------------------------------------------------------------- Init  
  
	/** Initializes the database.
	  * compiles SQL statements.
	  * @throws Exception if there is no connect to the database, or the precompile of SQL statements fails.
	  */
  public void init() throws Exception {
    if (!database.isOpened()) 
    	throw new Exception("SimpleVocAdmin.init(): Database is not open. Open Database before calling init().");

    //  add vocable
    final String sqlAddVoc =
          "INSERT INTO uservoc (userid, languageid1, languageid2,"
        + " vocable1, vocable2, state, date, unitid) "
        + "VALUES ( ? ,?, ?, ?, ?, ?, ?, ? ); ";

    // delete vocable
    final String sqlDelVoc =
          "DELETE * "
        + "FROM uservoc "
        + "WHERE vocid = ? AND userid = ? ; ";

    //  Edit vocable
    final String sqlEditVoc =
        "UPDATE uservoc "
        + "SET languageid1 = ?,languageid2 = ?,vocable1 = ?,"
        + "vocable2 = ?,date = ? "
        + "WHERE id = ? AND userid = ? ;";

    //  Edit vocablestate
    final String sqlEditVocState =
        "UPDATE uservoc "
        + "SET state = ? "
        + "WHERE id = ? AND userid = ? ;";

    // Adds a new Unit to database
    final String sqlAddUnit =
          "INSERT INTO unit ( unitname ) "
        + "VALUES ( ? ); ";

    // Deletes Unit
    final String sqlDelUnit =
          "DELETE * "
        + "FROM units "
        + "WHERE unitid = ? ; ";

    // Adds a new language to database
    final String sqlAddLanguage =
          "INSERT INTO languages ( languagename) "
        + "VALUES ( ? ); ";

    // Deletes Language
    final String sqlDelLanguage =
          "DELETE * "
        + "FROM languages "
        + "WHERE languageid = ? ; ";

    // Gets vocables by userid and language
    final String sqlGetVocByLangId =
        "SELECT vocid, languageid1, languageid2,"
        + " vocable1, vocable2, unitid "
        + "FROM uservoc "
        + "WHERE userid = ? AND (languageid1 = ? OR languageid2 = ?) ;";

    // Gets vocables by userid and language and date
    //  !!!!!!!! maybe Problem date ist String: String > String  = ?????
    final String sqlGetVocByDate =
        "SELECT vocid, languageid1, languageid2,"
        + " vocable1, vocable2, unitid "
        + "FROM uservoc "
        + "WHERE userid = ? AND (languageid1 = ? OR languageid2 = ?) AND date > ? ;";

    // Gets vocables by userid and VocId
    final String sqlGetVocByVocId =
        "SELECT vocid, languageid1, languageid2,"
        + " vocable1, vocable2, state, date, unitid "
        + "FROM uservoc "
        + "WHERE userid = ? AND vocid = ? ;";

    // Gets all LanguageNames NOT TESTED
    final String sqlGetAllLanguages =
        "SELECT languageid "
        + "FROM languages ;";

    // Gets all UnitNames NOT TESTED
    final String sqlGetAllUnits =
        "SELECT unitid "
        + "FROM units ;";

    // Gets langId out of LanguageName
    final String sqlGetLangIdByName =
        "SELECT languageid "
        + "FROM languages "
        + "WHERE languagename = ? ; ";

    // Gets unitid out of UnitName
    final String sqlGetUnitIdByName =
        "SELECT unitid "
        + "FROM units "
        + "WHERE unitname = ? ; ";

  // ------------------------------------------------- Precompile Statements
      try {
        preSQLAddVoc          =  database.precompile(sqlAddVoc);
        preSQLDelVoc          =  database.precompile(sqlDelVoc);
        preSQLEditVoc         =  database.precompile(sqlEditVoc);
        preSQLEditVocState    =  database.precompile(sqlEditVocState);
        preSQLAddUnit         =  database.precompile(sqlAddUnit);
        preSQLAddLanguage     =  database.precompile(sqlAddLanguage);
        preSQLDelUnit         =  database.precompile(sqlDelUnit);
        preSQLDelLanguage     =  database.precompile(sqlDelLanguage);
        preSQLGetVocByLangId  =  database.precompile(sqlGetVocByLangId);
        preSQLGetVocByDate    =  database.precompile(sqlGetVocByDate);
        preSQLGetVocByVocId   =  database.precompile(sqlGetVocByVocId);
        preSQLGetAllLanguages =  database.precompile(sqlGetAllLanguages);
        preSQLGetAllUnits     =  database.precompile(sqlGetAllUnits);
        preSQLGetLangIdByName =  database.precompile(sqlGetLangIdByName);
        preSQLGetUnitIdByName =  database.precompile(sqlGetUnitIdByName);

      } catch (SQLException exc) {
        throw new SQLException("SimpleVocUserAdmin: init() -> Precompileing SQL-Statements failed");
      }
  }


  // ------------------------------------------------------------------------------------ getVocable

	 /* Returns a vocable object from a user out of the database.
	  * The algorithm to look up the vocable object is determined by the modus.<br>
    * <b>0:</b> Randomly choosen out of the Unit-given Vocables.<br>
    * Units are spezified by the UserId = 1<br>
    * Using getVocByUserAndLanguage() and also
    * getVocableFromResultSetRandomly().<br>
    * Date can be null. Unit must be specified by Name.
    * using getUnitIdByUnitName() to get the UnitId.<br>
    * <b>1:</b> Randomly choosen out of all User-vocables by language.<br>
    * Using getVocByUserAndLanguage() and also
    * getVocableFromResultSetRandomly().<br>
    * Date and unit can be null.<br>
    * <b>2:</b> Randomly choosen out of all User-vocables by language and by
    * an algorithmen using the state of the Vocable-Object.<br>
    * Using getVocByUserAndLanguage() and also
    * using getVocableFromResultSetByStateRandomly().<br>
    * Date and unit can be null.<br>
    * <b>3:</b> Randomly choosen out of all User-vocables by language and by
    * by the oldest date of the Vocable-Object.<br>
    * Using getVocByUserAndLanguageAndDate() and also
    * using getVocableFromResultSetByDateRandomly().<br>
    * Date must be set. Unit can be null.<br>
    * ==================================================================<br>
	  * langId1 specifies the language from which the vocable object has to be translated to langId2.<br>
	  * The vocable has to be reorganised (see: <b>getResultSet()</b> )<br>
	  * @param user The owner of the vocables.
	  * @param langId1 The source language.
	  * @param langId2 The destination language to translate to.
	  * @param modus The mode that determines how to get the vocabluaries.
	  * @param date The oldest date to get the vocabluaries from.
	  * @param unitid The unit id to get the vocabluaries from.
	  * @return Returns a vocable object, if there were vocabluaries owned by <tt>user</tt>.<br>
	  *         <tt>null</tt> if there were no vocabluaries owned by this <tt>user</tt>.
	  *         Or there was no vocable "younger" than the given date.
	  * @throw InvalidArgumentException if <br>
	  * the modus was invalid.
	  * or the languages do not exists.
	  * or there is no unit found for <tt>unitid</tt>
	  * @throw SQLException if data couldn't be retrieved from the database.
	  *
	  * @todo HAS TO BE IMPLEMENTED
	  */
  public Vocable getVocable(User user,int langId1, int langId2, int modus, String date, int unitId) 
  throws IllegalArgumentException, SQLException {
    throw new SQLException("ERROR");
//    return new Vocable();
  }

	/** Returns the vocable object specified by the <tt>vocId</tt>.
	  * @param vocId The primary key of the vocable table in the database.
	  * @param user The owner of the vocable.
	  * @returns The vocable object specified by the <tt>vocId</tt>.
	  * @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED  
	  */
  public Vocable getVocable (int vocId) throws SQLException {
    return new Vocable();
  }

  // ------------------------------------------------------------------------------------ getVocablefrom ResultSets

  /** Returns a vocable object from a vector of vocable object, choosen randomly.
	  * @param resultSet A vector of vocable objects.
	  * @return A vocable object, choosen randomly from the given resultSet.
	  * @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public Vocable getVocableFromResultSetRandomly(java.util.Vector resultSet) throws SQLException {
    return new Vocable();
  }

	/** Returns a vocable object from a vector of vocable object,
	  * choosen randomly by an algorithmen using the state of the vocable object.
	  * @param resultSet A vector of vocable objects.
	  * @return A vocable object, choosen by an algorithmen from the given resultSet.
	  * @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public Vocable getVocableFromResultSetByStateRandomly(java.util.Vector resultSet) throws SQLException {
    return new Vocable();
  }

	/** Returns a vocable object from a vector of vocable object,
	  * choosen randomly by an algorithmen using the date of the vocable object.
	  * @param resultSet A set of vocable objects.
	  * @return A vocable object, choosen by an algorithmen from the given resultSet.
	  * @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public Vocable getVocableFromResultSetByDateRandomly(java.util.Vector resultSet) throws SQLException {
    return new Vocable();
  }

  // -------------------------------------------------------------------------------------------- generate ResultSets

  /** Returns a vector of vocable objects from the database, where <tt>userId</tt> and languages,
	  * given by the languageId, equals with the database.
	  * The vocable objects have to reorganised, so that <br>
	  * - Vocable.vocable1 is vocable in source language.<br>
	  * - Vocable.vocable2 is vocable in destination language..<br>
		* Calls <b>getResultSet(user,langid1,langid2,date)</b> with date = <tt>null</tt>.  
	  * @param user A User-Object.
	  * @param lang1 LanguageId of Vocable1.
	  * @param lang2 LanguageId of Vocable2.
	  * @return Returns a Vector of Vocable-Object.
	  * @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo THE REORGANISED VOCABLES HAVE TO BE SET IN THE VECTOR  
	  */
  public java.util.Vector getResultSet (User user, int lang1, int lang2, String date) throws SQLException {
    java.util.Vector returnVector = new java.util.Vector();
    if (date == null) {
      // Get ResultSet by User and language
      try {
        preSQLGetVocByLangId.setInt(1, user.getId());
        preSQLGetVocByLangId.setInt(2, lang1);
        preSQLGetVocByLangId.setInt(3, lang2);
        ResultSet result = preSQLGetVocByLangId.executeQuery();
        if( result.next() ) {
          Vocable voc = new Vocable();

          voc.setVocId( result.getInt(1) );
          voc.setLanguageId1( result.getInt(2) );
          voc.setLanguageId2( result.getInt(3) );
          voc.setVocable1( result.getString(4) );
          voc.setVocable2( result.getString(5) );
          voc.setUnitId( result.getInt(6) );
          returnVector.addElement(voc);
        } else {
          // alle Vocablen rausgezogen
          result.close();
          return returnVector;
        }

      } catch (SQLException exc) {
        throw new SQLException("SimpleVocAdmin.getResultSet(user,lang1,lang2,date): Cannot retrieve Vocables from database.");
      }

    } else {
      // Get ResultSet by User, language and date
      // MAYBE PROBLEMS WITH DATE CHECK IN DATABASE !!!!!!!!!!!!!!!!! USE WITH CAUTION !!!!!!!!!!!!
      try {
        preSQLGetVocByDate.setInt(1, user.getId());
        preSQLGetVocByDate.setInt(2, lang1);
        preSQLGetVocByDate.setInt(3, lang2);
        preSQLGetVocByDate.setString(4, date);
        ResultSet result = preSQLGetVocByLangId.executeQuery();
        if( result.next() ) {
          Vocable voc = new Vocable();

          voc.setVocId( result.getInt(1) );
          voc.setLanguageId1( result.getInt(2) );
          voc.setLanguageId2( result.getInt(3) );
          voc.setVocable1( result.getString(4) );
          voc.setVocable2( result.getString(5) );
          voc.setUnitId( result.getInt(6) );

          returnVector.addElement(voc);

        } else {
          // alle Vocablen rausgezogen
          result.close();
          return returnVector;
        }
      } catch (SQLException exc) {
        throw new SQLException("SimpleVocAdmin.getResultSet(user,lang1,lang2,date): Cannot retrieve Vocables from database with date.");
      }
    }

    return new  java.util.Vector();
  }

  // ------------------------------------------------------------------------------------ checkVocable

	/** Checks if the vocable is in the database.
	  * @param vocable The Vocable-Object to check.
	  * @param modus The current modus.
	  * @return Returns true if Vocable was answered correct, else false.
	  * @throws InvalidArgumentException if the modus was invalid.  
	  * @throws SQLException if data couldn't be retrieved from the database.
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean checkVocable(Vocable vocable, int modus) throws IllegalArgumentException, SQLException {
    return false;
  }


  // ------------------------------------------------------------------------------------ Unit Verwaltung

	/** Adds a unit to the database.
	  * The <tt>User.su_flag</tt> has to be checked to allow the adding.
	  * @param user To determine if user is superuser.
	  * @param unitname The name of the unit to add.
	  * @return Returns <tt>true</tt> if unitname was added, else <tt>false</tt>.
	  * @throws SQLException if data couldn't be addedd to the database.
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public  boolean addUnit(User user, String unitname) throws SQLException {
    return false;
  }

  /** Deletes a unit specified by <tt>unitId</tt> from the database.
	  * The <tt>User.su_flag</tt> has to be checked to allow the deleting.
	  * @param user To determine if user is superuser.
	  * @param unitId The id of the unit to delete.
	  * @return Returns <tt>true</tt> if unit was deleted, else <tt>false</tt>.
	  * @throws SQLException if data couldn't be deleted from the database.
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean delUnit(User user, int unitId) throws SQLException {
    return false;
  }
  
	/** Gets the <tt>UnitId</tt> by looking for the unitname in the Database.
	  * @param unitName The name of the choosen unit.
	  * @return Returns the <tt>unitid</tt> of the unit.
		* @throws InvalidArgumentException if the unitname does not exists.  
		* @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED		
	  */
  public int getUnitIdByUnitName(String unitName) throws /*IllegalArgumentException,*/ SQLException {
  	
    throw new SQLException("ERROR");
  }
  
	/** Gets all unitnames specified in the database.
	  * @return Returns a vector of Strings with all available unitnames.
		* @throws SQLException if data couldn't be retrieved from the database.
	  *
	  * @todo HAS TO BE IMPLEMENTED		
	  */
  public java.util.Vector getAllUnitNames() throws SQLException {
    return new java.util.Vector();
  }

  // ------------------------------------------------------------------------------------ Language Verwaltung

	/** Adds a language to the database.
	  * The <tt>User.su_flag</tt> has to be checked to allow the adding.
	  * @param user To determine if user is superuser.
	  * @param languagename The name of the language to add.
	  * @return Returns <tt>true</tt> if language was added, else <tt>false</tt>.
	  * @throws SQLException if data couldn't be added to the database.
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean addLanguage(User user, String languagename) throws SQLException {
    return false;
  }
  
	/** Deletes a language from the database.
	  * The <tt>User.su_flag</tt> has to be checked to allow the deleting.
	  * @param user To determine if user is superuser.
	  * @param languageId The id of the language to delete.
	  * @return Returns <tt>true</tt> if language was deleted, else <tt>false</tt>.
	  * @throws InvalidArgumentException if the language does not exists.
	  * @throws SQLException if data couldn't be deleted from the database.
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean delLanguage(User user, int languageId) throws SQLException {
    return false;
  }

	/** Returns a vector of all languages defined in the database.
	  * @return Returns a vector of Strings of all languages defined in the database.
	  * @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public java.util.Vector getAllLanguages() throws SQLException {
    return new java.util.Vector() ;
  }

	/** Returns the id of the language from the database.
	  * @param languagename The language.
	  * @return The id of the language.
	  * @throws InvalidArgumentException if the language does not exists.  
	  * @throws SQLException if data couldn't be retrieved from the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public int getLanguageIdByLanguagename(String languagename) throws /*IllegalArgumentException,*/ SQLException {
    return 0;
  }

  //------------------------------------------------------------------------------------- Vocable Verwaltung

	/** Adds a vocable object in the database with the owner given by the <tt>userId</tt> of the user.
	  * @param vocableSet This vocable object will be added in the database.
	  * @param user The owner of the vocableSet.
	  * @return <tt>true</tt> if adding was successfull, else throws an SQLException.
	  * @throws SQLException if data couldn't be added to the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean addVocable(Vocable vocableSet, User user) throws SQLException {
    return false;
  }

	/** Deletes a record of the database selected by the <tt>vocId</tt> of the vocable-object.
	  * Allowed to delete are owner and superuser.
	  * @param vocable This vocable object will be deleted in the database.
	  * @return <tt>true</tt> if deleting was successfull, else throws SQLException.
	  * @throws SQLException if data couldn't be deleted in the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean delVocable(Vocable vocable,User user) throws SQLException {
    return false;
  }

	/** Edits the <tt>state</tt> of a record in the database selected by the <tt>vocId</tt> of the vocable object.
	  * The <tt>state</tt> of the vocable object has to be changed by an algorithmen.
	  * @param vocableSet The <tt>state</tt> of this vocable object will be updated in the database.
	  * @param status If the answer was correct then status must be <tt>true</tt> else <tt>false</tt>.
	  * @param user The owner of the vocable.
	  * @return <tt>true</tt> if update was successfull, else throws an SQLException.
	  * @throws SQLException if data couldn't be updated in the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean editVocable(Vocable vocableSet, boolean status,User user) throws SQLException {
    return false;
  }

	/** Edits a record of the database selected by the <tt>vocId</tt> of the vocable object.
	  * @param vocableSet This vocable object will be updated in the database.
	  * @param user The owner of the vocable.
	  * @return <tt>true</tt> if update was successfull, <tt>false</tt> otherwise.
	  * @throws SQLException if data couldn't be updated in the database.  
	  *
	  * @todo HAS TO BE IMPLEMENTED	  
	  */
  public boolean editVocable(Vocable vocableSet,User user) throws SQLException  {
    return false;
  }
}
