/********************************************************************
  *
  *  interface   : VocAdmin.java
  *  author      : Karsten Wolke, Niels-Peter de Witt
  *  version     : 2000-11-28(001)
  *  (c)         : Niels-Peter de Witt
  *  email       : ndewitt@gmx.de
  *  description : Handles the vocable administration with a database.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * -----------------------------------------------------------------
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
********************************************************************/

package de.janicke.hj.tools.servlet;

import java.sql.*;


/** Handles the vocable administration with a Database.
  */
public interface VocAdmin {
	
// ----------------------------------------------------------------------------------- Init

/** Initializes the database.
  * compiles SQL statements.
  * @return true if succeeded,<br>
  *					if the database is not opened or the compilation of the SQL statements fails, it throws an Exception.
  * @throws Exception if there is no connect to the database, or the precompile of SQL statements fails.
  */
public void init() throws Exception;

// ------------------------------------------------------------------------------------ getVocable

/** Returns a vocable object from a user out of the database.
  * The algorithm to look up the vocable object is determined by the modus.<br>
  * langId1 specifies the language from which the vocable object has to be translated to langId2.<br>
  * The vocable has to be reorganised (see: <b>getResultSet()</b> )<br>
  * @param user The owner of the vocables.
  * @param langId1 The source language.
  * @param langId2 The destination language to translate to.
  * @param modus The mode that determines how to get the vocabluaries.
  * @param date The oldest date to get the vocabluaries from.
  * @param unitid The unit id to get the vocabluaries from.
  * @return Returns a vocable object, if there were vocabluaries owned by <tt>user</tt>.<br>
  *         <tt>null</tt> if there were no vocabluaries owned by this <tt>user</tt>.
  *         Or there was no vocable "younger" than the given date.
  * @throw InvalidArgumentException if <br>
  * the modus was invalid.
  * or the languages do not exists.
  * or there is no unit found for <tt>unitid</tt>
  * @throw SQLException if data couldn't be retrieved from the database.
  */
public Vocable getVocable(User user,int langId1, int langId2, int modus, String date, int unitId) 
throws IllegalArgumentException, SQLException;

/** Returns a vocable object from a user out of the database.
  * How to get the vocable object is given by the modus.<br>
  * It gets the languageId's out of lang1 and lang2
  * by using <tt>getLanguageByIdLanguagename()</tt> an then calls
  * getVocable() with the languageId's.<br>
  * The vocable has to be reorganised (see: <b>getResultSet()</b> )<br>
  * @param user The owner of the vocables.
  * @param lang1 The source language.
  * @param lang2 The destination language to translate to.
  * @param modus The mode that determines how to get the vocabluaries.
  * @param date The oldest date to get the vocabluaries from.
  * @param unitid The unit id to get the vocabluaries from.
  * @return Returns a vocable object, if there were vocabluaries owned by <tt>user</tt>.<br>
  *         <tt>null</tt> if there were no vocabluaries owned by this <tt>user</tt>.
  *         Or there was no vocable "younger" than the given date.
  * @throws InvalidArgumentException if <br>
  * the modus was invalid.
  * or the languages do not exists.
  * or there is no unit found for <tt>unitid</tt>
  * @throws SQLException if data couldn't be retrieved from the database.
  */
public Vocable getVocable(User user, String lang1, String lang2, int modus, String date, int unitId) 
throws IllegalArgumentException, SQLException;

/** Returns a vocable object from a user out of the database.
  * How to get the vocable object is given by the modus.<br>
  * It gets the languageId's out of lang1 and lang2
  * by using getLanguageByIdLanguagename() and
  * getting the <tt>unitId</tt> by using getUnitIdByUnitName().
  * Then calls getVocable() with the languageId's.<br>
  * The vocable has to be reorganised (see: <b>getResultSet()</b> )<br>
  * @param user The owner of the vocables.
  * @param langId1 The source language.
  * @param langId2 The destination language to translate to.
  * @param modus The mode that determines how to get the vocabluaries.
  * @param date The oldest date to get the vocabluaries from.
  * @param unit The unit to get the vocabluaries from.
  * @return Returns a vocable object, if there were vocabluaries owned by <tt>user</tt>.<br>
  *         <tt>null</tt> if there were no vocabluaries owned by this <tt>user</tt>.
  *         Or there was no vocable "younger" than the given date.
  * @throws InvalidArgumentException if <br>
  * the modus was invalid.
  * or the languages do not exists.
  * or there is no unit found for <tt>unit</tt>
  * @throws SQLException if data couldn't be retrieved from the database.
  */
public Vocable getVocable(User user,int langId1, int langId2, int modus, String date, String unit) 
throws IllegalArgumentException, SQLException;

/** Returns the vocable object specified by the <tt>vocId</tt>.
  * @param vocId The primary key of the vocable table in the database.
  * @param user The owner of the vocable.
  * @returns The vocable object specified by the <tt>vocId</tt>.
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public Vocable getVocable(int vocId) throws SQLException;

// ------------------------------------------------------------------------------------ getVocablefrom ResultSets

/** Returns a vocable object from a vector of vocable object, choosen randomly.
  * @param resultSet A vector of vocable objects.
  * @return A vocable object, choosen randomly from the given resultSet.
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public Vocable getVocableFromResultSetRandomly(java.util.Vector resultSet) throws SQLException;

/** Returns a vocable object from a vector of vocable object,
  * choosen randomly by an algorithmen using the state of the vocable object.
  * @param resultSet A vector of vocable objects.
  * @return A vocable object, choosen by an algorithmen from the given resultSet.
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public Vocable getVocableFromResultSetByStateRandomly(java.util.Vector resultSet) throws SQLException;

/** Returns a vocable object from a vector of vocable object,
  * choosen randomly by an algorithmen using the date of the vocable object.
  * @param resultSet A set of vocable objects.
  * @return A vocable object, choosen by an algorithmen from the given resultSet.
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public Vocable getVocableFromResultSetByDateRandomly(java.util.Vector resultSet) throws SQLException;


// -------------------------------------------------------------------------------------------- generate ResultSets

/** Returns a vector of vocable objects from the database, where <tt>userId</tt> and languages,
  * given by the languageId, equals with the database.
  * @param user A User-Object.
  * @param lang1 LanguageId of Vocable1.
  * @param lang2 LanguageId of Vocable2.
  * @return Returns a Vector of Vocable-Object.
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public java.util.Vector getResultSet(User user, int lang1, int lang2 ) throws SQLException;

/** Returns a vector of vocable objects from the database, where <tt>userId</tt> and languages,
  * given by the language, equals with the database.
  * The vocable objects have to reorganised, so that 
  * - Vocable.vocable1 is vocable in source language.<br>
  * - Vocable.vocable2 is vocable in destination language..<br>
  * @param user A user object.
  * @param langName1 Languagename of vocable1.
  * @param langName2 Languagename of vocable2.
  * @return Returns a vector of vocable object.
	* @throws InvalidArgumentException if the at least one of the languages does not exists.    
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public java.util.Vector getResultSet(User user, String langName1, String langName2 ) 
throws IllegalArgumentException, SQLException;

/** Returns a vector of vocable objects from the database, where <tt>userId</tt>, languages
  * given by the languageId equals with the database and where the date of the vocable is younger than the given date.
  * The vocable objects have to reorganised, so that 
  * - Vocable.vocable1 is vocable in source language.<br>
  * - Vocable.vocable2 is vocable in destination language..<br>
  * @param user A user object.
  * @param lang1 LanguageId of vocable1.
  * @param lang2 LanguageId of vocable2.
  * @param date The date of the oldest vocable that will be returned.
  * @return Returns a Vector of Vocable-Object.
  * @throws SQLException if data couldn't be retrieved from the database.    
  */
public java.util.Vector getResultSet(User user, int lang1, int lang2, String date) throws SQLException;

/** Returns a vector of vocable objects from the database, where <tt>userId</tt>, languages
  * given by the language equals with the database and where the date of the vocable is younger than the given date.
  * The vocable objects have to reorganised, so that 
  * - Vocable.vocable1 is vocable in source language.<br>
  * - Vocable.vocable2 is vocable in destination language..<br>
  * @param user A user object.
  * @param langName1 Languagename of vocable1.
  * @param langName2 Languagename of vocable2.
  * @param date The date of the oldest vocable that will be returned.
  * @return Returns a vector of vocable object.
	* @throws InvalidArgumentException if the at least one of the languages does not exists.  
	* @throws SQLException if data couldn't be retrieved from the database.    
  */
public java.util.Vector getResultSet (User user, String langName1, String langName2, String date ) 
throws IllegalArgumentException, SQLException;

// ------------------------------------------------------------------------------------ checkVocable

/** Checks if the vocable is in the database.
  * @param vocable The Vocable-Object to check.
  * @param modus The current modus.
  * @return Returns true if Vocable was answered correct, else false.
  * @throws InvalidArgumentException if the modus was invalid.  
  * @throws SQLException if data couldn't be retrieved from the database.
  */
public boolean checkVocable(Vocable vocable, int modus) throws IllegalArgumentException, SQLException;

// ------------------------------------------------------------------------------------ Unit Verwaltung

/** Adds a unit to the database.
  * The <tt>User.su_flag</tt> has to be checked to allow the adding.
  * @param user To determine if user is superuser.
  * @param unitname The name of the unit to add.
  * @return Returns <tt>true</tt> if unitname was added, else <tt>false</tt>.
  * @throws SQLException if data couldn't be addedd to the database.
  */
public boolean addUnit(User user, String unitname) throws SQLException;

/** Deletes a unit from the database.
  * The <tt>User.su_flag</tt> has to be checked to allow the deleting.
  * @param user To determine if user is superuser.
  * @param unitname The name of the unit to delete.
  * @return Returns <tt>true</tt> if unit was deleted, else <tt>false</tt>.
  * @throws InvalidArgumentException if the unitname does not exists.
  * @throws SQLException if data couldn't be deleted from the database.
  */
public boolean delUnit(User user, String unitname) throws IllegalArgumentException, SQLException;

/** Deletes a unit specified by <tt>unitId</tt> from the database.
  * The <tt>User.su_flag</tt> has to be checked to allow the deleting.
  * @param user To determine if user is superuser.
  * @param unitId The id of the unit to delete.
  * @return Returns <tt>true</tt> if unit was deleted, else <tt>false</tt>.
  * @throws SQLException if data couldn't be deleted from the database.
  */
public boolean delUnit(User user, int unitId) throws SQLException;

/** Gets the <tt>UnitId</tt> by looking for the unitname in the Database.
  * @param unitName The name of the choosen unit.
  * @return Returns the <tt>unitid</tt> of the unit.
	* @throws InvalidArgumentException if the unitname does not exists.  
	* @throws SQLException if data couldn't be retrieved from the database.  
  */
public int getUnitIdByUnitName(String unitName) throws IllegalArgumentException, SQLException;

/** Gets all unitnames specified in the database.
  * @return Returns a vector of Strings with all available unitnames.
	* @throws SQLException if data couldn't be retrieved from the database.
  */
public java.util.Vector getAllUnitNames() throws SQLException;

// ------------------------------------------------------------------------------------ Language Verwaltung

/** Adds a language to the database.
  * The <tt>User.su_flag</tt> has to be checked to allow the adding.
  * @param user To determine if user is superuser.
  * @param languagename The name of the language to add.
  * @return Returns <tt>true</tt> if language was added, else <tt>false</tt>.
  * @throws SQLException if data couldn't be added to the database.
  */
public boolean addLanguage(User user, String languagename) throws SQLException;

/** Deletes a language from the database.
  * The <tt>User.su_flag</tt> has to be checked to allow the deleting.
  * @param user To determine if user is superuser.
  * @param languagename The name of the language to delete.
  * @return Returns <tt>true</tt> if language was deleted, else <tt>false</tt>.
  * @throws InvalidArgumentException if the language does not exists.
  * @throws SQLException if data couldn't be deleted from the database.
  */
public boolean delLanguage(User user, String languagename) throws IllegalArgumentException, SQLException;

/** Deletes a language from the database.
  * The <tt>User.su_flag</tt> has to be checked to allow the deleting.
  * @param user To determine if user is superuser.
  * @param languageId The id of the language to delete.
  * @return Returns <tt>true</tt> if language was deleted, else <tt>false</tt>.
  * @throws InvalidArgumentException if the language does not exists.
  * @throws SQLException if data couldn't be deleted from the database.
  */
public boolean delLanguage(User user, int languageId) throws SQLException;

/** Returns a vector of all languages defined in the database.
  * @return Returns a vector of Strings of all languages defined in the database.
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public java.util.Vector getAllLanguages() throws SQLException;

/** Returns the id of the language from the database.
  * @param languagename The language.
  * @return The id of the language.
  * @throws InvalidArgumentException if the language does not exists.  
  * @throws SQLException if data couldn't be retrieved from the database.  
  */
public int getLanguageIdByLanguagename(String languagename) throws IllegalArgumentException, SQLException;

//------------------------------------------------------------------------------------- Vocable Verwaltung

/** Edits a record of the database selected by the <tt>vocId</tt> of the vocable object.
  * @param vocableSet This vocable object will be updated in the database.
  * @param user The owner of the vocable.
  * @return <tt>true</tt> if update was successfull, <tt>false</tt> otherwise.
  * @throws SQLException if data couldn't be updated in the database.  
  */
public boolean editVocable(Vocable vocableSet,User user) throws SQLException;

/** Adds a vocable object in the database with the owner given by the <tt>userId</tt> of the user.
  * @param vocableSet This vocable object will be added in the database.
  * @param user The owner of the vocableSet.
  * @return <tt>true</tt> if adding was successfull, else throws an SQLException.
  * @throws SQLException if data couldn't be added to the database.  
  */
public boolean addVocable(Vocable vocableSet, User user) throws SQLException;

/** Deletes a record of the database selected by the <tt>vocId</tt> of the vocable-object.
  * Allowed to delete are owner and superuser.
  * @param vocable This vocable object will be deleted in the database.
  * @return <tt>true</tt> if deleting was successfull, else throws SQLException.
  * @throws SQLException if data couldn't be deleted in the database.  
  */
public boolean delVocable(Vocable vocable, User user) throws SQLException;

/** Edits the <tt>state</tt> of a record in the database selected by the <tt>vocId</tt> of the vocable object.
  * The <tt>state</tt> of the vocable object has to be changed by an algorithmen.
  * @param vocableSet The <tt>state</tt> of this vocable object will be updated in the database.
  * @param status If the answer was correct then status must be <tt>true</tt> else <tt>false</tt>.
  * @param user The owner of the vocable.
  * @return <tt>true</tt> if update was successfull, else throws an SQLException.
  * @throws SQLException if data couldn't be updated in the database.  
  */
public boolean editVocable(Vocable vocableSet, boolean status,User user) throws SQLException;

}
