/********************************************************************
  *
  *  program     : VocTrainer.java
  *  author      : Helge Janicke
  *  version     : 2000-12-25(001)
  *  (c)         : Helge Janicke
  *  email       : heljanic@agent.fho-emden.de
  *  description : VocTrainer is a Servlet. This servlet is written to
  *   run with the vq server on windows platforms, but it should run on
  *   other servers as well. For a detailed description please read the
  *   provided JavaDoc documentation.
  *
  *******************************************************************
  * modifications:
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2000-12-25    heljanic    Build in localization.
  *                           this is done using a PropertyResourceBundle
  *                           named rb.
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
********************************************************************/

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import de.janicke.hj.tools.servlet.*;

/** VocTrainer is a servlet. It is designed to run with the vq-server on
Windows&nbsp; platforms. It may also run with other servers on different
platforms as well, but this has not been tested, yet.</p>
<p>VocTrainer is a (as one can guess from the name) vobable trainer. For more
detailed information on the performance and abilities of this software please
read the (TeX, pdf) documentation that goes with this distribution. The
following text describes the general concept and the implementation of this
servlet.</p>
<p>The two commonly used methods in a servlet are the doPost(..) and doGet(..)
methods. We combined these two methods to a simple doMain(..) methods for we do
not really need to differ between post and get. The doMain method handles the
user administration stuff that is needed for most servlets. If the servlet is
called the first time by a user there is no event specified and no HTML page
supplied. In this case the login HTML template is shown. The other actions that
are provided by this method are deleting a user and generating a new account.
All other events are delegated to the handleEvent(...) method of this class.</p>
<p>There is no direct access to the database that holds the information of the
users. All database access is shielded by either the UserAdministration class
(for user information) or VocableAdministration class (for vocable information).
If you are interested in the database connection and handling please read the
documentation of these two classes.</p>
<p>To be more flexible in displaying HTML pages we use HTML template files. A
template file is a file which contains most of the information but also leaves
some gaps to fill in case dependend data, such a a nickname or the secure
identity of a user. The template files are handled using the HTMLTemplate class.
To understand the following concepts you should be familiar with the HTMTemplate
class and the basic methods of the UserAdministration.</p>
<p>The base concept is that there is a HTML form which at least has three
fields:</p>
<ol>
  <li>event: The event that should be handled. E.g. &quot;login&quot;.</li>
  <li>id: The id of the user.</li>
  <li>secureidentity: A random String that is given to a user when she logs in.
    This key is unique during a session.</li>
</ol>
<p>If there is no event specified then the login HTML template is shown by
default. If there is a event specified that is not handled by the servlet an
error HTML template is shown. In all other cases the code block for the
eventhandling is responsible for the following pages displayed. Any uncaught
Exception will also force to display the error page.</p>
<p>The steps in the eventhandling:</p>
<ol>
  <li>clear all variables from the previous session.</li>
  <li>check if there is an event. If no event show login template</li>
  <li>retrieve id and secureidentity. Used to authorize a user. If a user is not
    logged in, these fields are empty.</li>
  <li>handle specified event. If either login, delete or&nbsp; newuser.</li>
  <li>if there is an other event check the users id and secureidentity. This
    uses the UserAdministration.getUser(..) method. If the user can be
    authorized the event handling is delegated to the &quot;inner side event
    handling&quot; of the servlet: handleEvents(...).</li>
  <li>set the id and the secureidentity of the authorized user for the next
    template. It is important never to loose this information when displaying a
    HTML page. The user will loose her authorization!</li>
  <li>build up the next HTML template (depends on the specified event). Means
    fill the gaps in the template.</li>
  <li>send the HTML file back to the user.</li>
  <li>wait for the next request.</li>
</ol>
<p>Dear subclassers:</p>
<p>It' s possible to subclass this servlet to take advantage of the already
build login procedure. Please ensure when you plan to do so that you overwrite
the handleEventMethod of this class. In this case do call the the method before
you do your own eventhandling. A returnvalue null means that the event was not
handled. Any other returnvalue should be taken as result and be returned! Please
read the description of the method carefully. The second important thing is that
you do use the same user fields in your database as they are used in the sample
database distributed with this package. See the UserAdministration class for
more detailed information of the database structure.</p>
<p>DO NOT USE THE FOLLOWING EVENTS:</p>
<ul>
  <li>login</li>
  <li>deleteuser</li>
  <li>logout</li>
  <li>edit</li>
  <li>user</li>
  <li>newuser</li>
  <li>calldeleteuser</li>
</ul>
*/
public class VocTrainer extends HttpServlet {

  /** current database */
  private Database database = new Database();

  /** user administration */
  protected SimpleUserAdministration userAdmin = new SimpleUserAdministration(database);

  /** holds the servlet parameters of the configuration,
    * e.g. the name of the servlet or the real path.
    */
  protected ServletParameters servletParam = null;

  /** holds the variables and HTML template files. */
  protected HTMLTemplate template = new HTMLTemplate();

  /** the path for the login template. */
  private String login;

  /** the path for the start template. */
  private String start;

  /** the path for the newuser template. */
  private String newuser;

  /** the path for the deleteuser template. */
  private String deleteuser;

  /** the path for the edit user template. */
  private String edituser;

  /** the path for the error template. */
  private String error;

  /** the locale that is used to display the templates and messages. */
  protected Locale locale = new Locale("de", "DE");

  /** the ResourceBundle for this servlet. */
  protected ResourceBundle rb = ResourceBundle.getBundle("ServletResource", locale);

  /** initializes this servlet */
  public void init( ServletConfig in_conf ) throws ServletException {
    super.init( in_conf );
    log( "--- init ---" );

    if (database.open()) {
      log("------ database is now initialized");
    } else {
      log("------ failed to open database,");
      System.exit(0);
    }

    try {
      userAdmin.init();
      log("------ User Administration is now initialized");
    } catch (Exception exc) {
      log("------ failed to initialize user administration.");
      System.exit(0);
    }

    servletParam = new ServletParameters(this);

    setDefaults();

    // set destination of default HTML template-files.
    login = servletParam.getTemplatePath().getAbsolutePath() + "\\index.html";
    start = servletParam.getTemplatePath().getAbsolutePath() + "\\start.html";
    newuser = servletParam.getTemplatePath().getAbsolutePath() + "\\user_neu.html";
    deleteuser = servletParam.getTemplatePath().getAbsolutePath() + "\\user_loeschen.html";
    edituser = servletParam.getTemplatePath().getAbsolutePath() + "\\user_bearbeiten.html";
    error = servletParam.getTemplatePath().getAbsolutePath() + "\\error.html";

    log("--- end of init ---");
  }

  /** Initializes the default values for variables.
    * This method is called once when initializing the servlet.
    */
  private void setDefaults() {
    template.putDefault("errormessage","no message specified");
    template.putDefault("errorcode","no error code specified");
    template.putDefault("errorcontext","no error context specified");
    template.putDefault("errorstring","not specified");

    template.putDefault("stylesheetnormal", rb.getString("stylesheetnormal"));
    template.putDefault("help", rb.getString("help"));

    // initialize the variables and their values.
    template.putDefault("servlet", servletParam.getServletName());
    template.putDefault("nickname", "");
    template.putDefault("surename", "");
    template.putDefault("forename", "");
    template.putDefault("email", "");
    template.putDefault("secureidentity", "");
    template.putDefault("info", "");
    template.putDefault("id", "");

    // images
    template.putDefault("imglogo", rb.getString("imglogo"));
    template.putDefault("imghelp", rb.getString("imghelp"));
    template.putDefault("imgcollage", rb.getString("imgcollage"));

  }

  /** Combines the doGet() and doPost() methodes.
    * Handles all requests and responses
    * This method body does actually the whole user administration.
    * @param request the HttpServletRequest given by either doGet() or doPost().
    * @param response the HttpServletResponse given by either doGet() or doPost().
    * @throws ServletException
    * @throws IOException
    */
  private void doMain(HttpServletRequest request, HttpServletResponse response)
    throws ServletException,IOException
  {

    // standard output
    String output=null;

    // stores the event variable
    String event=null;

    // stores the secure identity identification number
    String secureIdentity=null;

    // stores the user id.
    String id=null;
    int intId;

    // the current user which is given to each event after he's authorized.
    User currentUser = null;

    // Clears all variables set by the previous call. Defaults are not affected!
    template.clearVariables();

    // ??????????????????????????????????????????????
    //locale = ((ServletRequest) request).getLocale();

    try {
      // loads the resourcebundle for the specified locale
      rb = ResourceBundle.getBundle("ServletResource", locale);

      // retrieve the event, secureIdentity and id parameters.
      // !!!!!!!!!!!!!!!!!!!!!!!!! use only this values for later checks
      // It seems that a second call might not return the same results - a bug?
      // took me some hours to find that one!
      event = request.getParameter("event");
      secureIdentity = request.getParameter("secureidentity");
      id = request.getParameter("id");

      // setting the correct helpfile
      template.putDefault("help", rb.getString("help"));


//      log("event = " + event);
//      log("---- id = " + id );
//      log("---- secureIdentity = " + secureIdentity);


      if (event != null) {
        // there is a Event specified so lets have a look what is to be done.

/**************************************************************************
LOGIN
***************************************************************************/

          if (event.equals("login")) {
        // it's the login event
//          log("login");

          // retrieve nick and password
          String nickname = request.getParameter("nickname");
          String password = request.getParameter("password");

          // try to login the user. currentUser == null means login failed.
          // See UserAdministration for more details.
          currentUser = userAdmin.loginUser(nickname, password, request);

//          log("current user = " + currentUser);

          if (currentUser != null) {
            // user is authorized now.
//			System.err.println("login : ");
//            System.err.println("id = " + currentUser.getId() + " secure.. = "+ currentUser.getSecureIdentity() );
            template.put("id",""+ currentUser.getId() );
            template.put("secureidentity", currentUser.getSecureIdentity());

            String file = template.loadFile(start, locale);
            output = template.buildTemplate(file);
          } else {
            // login failed
            template.put("nickname", nickname);
            template.put("info", rb.getString("loginFailed"));

            String file = template.loadFile(login, locale);
            output = template.buildTemplate(file);
          }

/**************************************************************************
NEWUSER
***************************************************************************/

        } else if (event.equals("newuser")) {
          // new user entry.
//          log("new user");

          String file = template.loadFile(newuser, locale);
          output = template.buildTemplate(file);


//		  System.err.println("newuser = " + newuser );
//          output = template.makeTemplate(newuser);
//		  System.err.println("here's newuser demand returned from makeTemplate = " + output);
/**************************************************************************
DELETEUSER
***************************************************************************/

        } else if (event.equals("deleteuser")) {

          String nickname = request.getParameter("nickname");
          String password = request.getParameter("password");

          currentUser = userAdmin.loginUser(nickname, password, request);

        if (currentUser != null) {
            userAdmin.deleteUser( currentUser );
            template.put("info", rb.getString("userDeleteConfirmed"));
            template.put("id","");
            String file = template.loadFile(login, locale);
            output = template.buildTemplate(file);
          } else {
            template.put("info", rb.getString("userDeleteRejected"));
            template.put("nickname", nickname);

            String file = template.loadFile(deleteuser, locale);
            output = template.buildTemplate(file);
          }

/**************************************************************************
USER
***************************************************************************/

        } else if (event.equals("user")) {
          // user signed a new account.

          // generating an empty User object.
          User newUser = new User();

          // retrieve information from HTML
          String surename = request.getParameter("surename");
          String forename = request.getParameter("forename");
          String nickname = request.getParameter("nickname");
          String email = request.getParameter("email");
          String password = request.getParameter("password");
          String passwordcontrol = request.getParameter("passwordcontrol");

          // apply this information to the new User object.
          newUser.setSurename(surename);
          newUser.setForename(forename);
          newUser.setNickname(nickname);
          newUser.setEmail(email);
          newUser.setPassword(password);

          // check if control password equals the given passsword.
          if (password.equals(passwordcontrol)) {
            // equals

            // try to add the new User to database.
            // see UserAdministration for more details.
            if (userAdmin.addUser( newUser ) ) {
              // succeeded

              // login user. currentUser == null means login failed
              currentUser = userAdmin.loginUser(newUser.getNickname(), newUser.getPassword(), request);

              // check if user in logged in
              if (currentUser != null) {
                // user is authorized now.

//				System.err.println("current user after loginUser = " + currentUser);

                template.put("id",""+ currentUser.getId() );
                template.put("secureidentity", currentUser.getSecureIdentity());

                String file = template.loadFile(start, locale);
                output = template.buildTemplate(file);

              } else {
                // login failed

                template.put("errorstring", "new user cannot login.");
                template.put("errorcontext", "new user");
                template.put("errormessage", "Voctrainer failed in creating a new user.");
                output = buildErrorTemplate();
              }
            } else {
              // addUser(...) failed.
              // the nickname does already exist.

              // Put given information to the new user template.
              template.put("info", rb.getString("nickAlreadyExists"));
              template.put("nickname", "");
              template.put("surename", surename);
              template.put("forename", forename);
              template.put("email", email);

              // show this template again.
              String file = template.loadFile(newuser, locale);
              output = template.buildTemplate(file);
            }

          } else {
            // control password does not equal password.

            // Put given information to the new user template.
            template.put("info", rb.getString("ctrlPWDNotPWD"));
            template.put("nickname", nickname);
            template.put("surename", surename);
            template.put("forename", forename);
            template.put("email", email);

            // show this template again.
            String file = template.loadFile(newuser, locale);
            output = template.buildTemplate(file);
          }

/**************************************************************************
CALLDELETEUSER
***************************************************************************/

        } else if (event.equals("calldeleteuser") ) {
          // page for deleting users.
//          log("delete user");

          String file = template.loadFile(deleteuser, locale);
          output = template.buildTemplate(file);

/**************************************************************************
DELEGATE THE EVENT TO HANDLEEVENT
***************************************************************************/

        } else {
          // other events delegated to handleEvent.
          // String id = "error id";
          // String secureIdentity = "error secure identity";

          try {
            // retrieve id and secureidentity from HTML
            // id = request.getParameter("id");
            // secureIdentity = request.getParameter("secureidentity");

            // put this information to the template for the next template to build.
            template.put("id","" + id );
            template.put("secureidentity", secureIdentity);

            // try to parse id into int.
            intId = Integer.parseInt(id);
          } catch ( NumberFormatException ex) {
            // no good to have an id which is not an int !!!
            throw new NumberFormatException("id =" + id + " was not a number");
          }

          // retrieve the user from the id and secureidentity.
          // no user matching this two parameters null is returned.

//			System.err.println("checking user authentification intId = " + intId + " secure.. = " + secureIdentity);

          currentUser = userAdmin.getUser(intId, secureIdentity);
//			System.err.println(" delegate event : currentuser = " + currentUser);
          // this is to store the returnvalue from handleEvents(...)
          String handle = null;

          if (currentUser != null) {
            // if user is authorized then delegate event.
            handle = handleEvents(request, response, currentUser, event);

            // put this information to the template for the next template to build.
            template.put("id",""+ currentUser.getId() );
            template.put("secureidentity", currentUser.getSecureIdentity());
          } else {
            // user wasn't authorized
            template.put("errormessage","You were not authorized!");
            template.put("errorcode", "doMain @ call handleEvents");
            template.put("errorcontext", "Tried to enter inner sites where identification is needed");
            throw new Exception("unauthorized attempt to access inner sites.");
          }

          // check if there was a returnvalue from handleEvents(...)
          if (handle != null) {
            // got response from handleEvent.
              output = handle;
          } else {
            // event wasn't handled.
            template.put("nickname", currentUser.getNickname());
            template.put("errormessage","Event is not handled!");
            template.put("errorcode", "none");
            template.put("errorcontext", "doMain @ call handleEvents try to set output");
            throw new Exception("event ["+ event +"] is not supported");
          }
        }
      } else {

/**************************************************************************
NO EVENT SPECIFIED IN HTML
***************************************************************************/
        // no event is specified -> show login

        template.put("info", rb.getString("welcome"));
        template.put("nickname", "");
        template.put("password", "");

        String file = template.loadFile(login, locale);
        output = template.buildTemplate(file);
      }
    } catch( Exception exc ) {
      // something went wrong!!!
      template.put("errorstring", exc.toString() );
      output = buildErrorTemplate();
      exc.printStackTrace();
    }

    // sent the output back to browser.
    response.setContentType("text/html");
    PrintWriter out = response.getWriter();
    out.println(output);
    out.flush();
  }


  /** Method to build up the error template.
    * If there occurs any exception in this Method it will be caught and
    * treated as fatal. In this case there will be an output to the log stream of the server
    * and a stack trace is written to stderr.
    * @return the error html page.
    */
  private String buildErrorTemplate() {
    String rv="ERROR!";
    try{

      String file = template.loadFile(error, locale);
      rv = template.buildTemplate(file);
    } catch (Exception exc) {
    exc.printStackTrace();
    log(" !!!!! fatal exception during doMain, error building error template");
    log(" !!!!! " + exc.getMessage());
    rv = "error building error template" + exc.getMessage();
    }
    return rv;
  }


  // Overwritten from HttpServlet.
  /** Delegates to doMain(...)
    */
  public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException,IOException
  {
    doMain(request, response);
  }

  // Overwritten from HttpServlet.
  /** Delegates to doMain(...)
    */
  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException,IOException
  {
    doMain(request, response);
  }

  /** Handles all events of the actual servlet implementation except the user administration and login.
    * To add a additional event please insert a new <br><tt>else if ( event.equals("YOUR EVENT") )</tt>
    * block where your event is handled. Please take care of other events already specified if you're
    * adding events to an existing servlet.
    * If you're writing a new servlet using this one as a back bone. please subclass and overwrite this
    * method with your own events.<br>
    * IMPORTANT: Do not call template.clear() or template.clearVariables() method from this method.
    * If you do ensure to copy the secureidentity and id variables into the new variables hash! If you
    * do not care about this, the user will loose her authorization!
    * @param request The request from doMain().
    * @param response The response from doMain().
    * @param user The authorized user.
    * @param event The event that should be handled.
    * @return The already build template file as a String. This string is sent directly to the browser,
    * without any further modifications. If the given event is not handled then this method should
    * return null.
    * @throw any Exception that can occure during event handling. To support quick error trapping please
    * set the variable "errormessage", "errorcode" and "errorcontext" before throwing a exception.
    * In case of an thrown exception the error.html template is sent back.
    */
  protected String handleEvents(HttpServletRequest request, HttpServletResponse response, User user, String event)
  throws Exception {
    String returnValue=null;

/**************************************************************************
CALLEDITUSER
***************************************************************************/

    if ( event.equals("calledituser") ) {
      template.put("nickname", user.getNickname());
      template.put("surename", user.getSurename());
      template.put("forename", user.getForename());
      template.put("email", user.getEmail());

      String file = template.loadFile(edituser, locale);
      returnValue = template.buildTemplate(file);
      return returnValue;

/**************************************************************************
EDITUSER
***************************************************************************/

    } else if ( event.equals("edituser")) {
      String nickname = request.getParameter("nickname");
      String surename = request.getParameter("surename");
      String forename = request.getParameter("forename");
      String email = request.getParameter("email");
      String passwd = request.getParameter("password");
      String passwdctrl = request.getParameter("passwordcontrol");
      boolean proceed=true;

      if (passwd.equals(passwdctrl) && passwd.length() > 0) {
        try {
          userAdmin.editUser(user, passwd);
          // passwd approved.
        } catch (Exception exc) {
          // passwd failed. To simple.
          proceed = false;
          template.put("info", rb.getString("edit_tooSimplePWD"));
          template.put("nickname", user.getNickname());
          template.put("surename", user.getSurename());
          template.put("forename", user.getForename());
          template.put("email", user.getEmail());

          String file = template.loadFile(edituser, locale);
          returnValue = template.buildTemplate(file);
        }
      } else {
        // passwd != passwdchange. zerolength means no change.
        if (passwd.length() != 0) {
          proceed = false;
          template.put("info", rb.getString("edit_PWDNotControlPWD"));
          template.put("nickname", user.getNickname());
          template.put("surename", user.getSurename());
          template.put("forename", user.getForename());
          template.put("email", user.getEmail());

          String file = template.loadFile(edituser, locale);
          returnValue = template.buildTemplate(file);
        }
      }

      if (proceed) {
        if ((!user.getNickname().equals(nickname)) && (userAdmin.getUser(nickname) != null)) {
          // nickname already exist.
          proceed = false;
          template.put("info", rb.getString("edit_NickAlreadyExists"));
          template.put("surename", user.getSurename());
          template.put("forename", user.getForename());
          template.put("email", user.getEmail());

          String file = template.loadFile(edituser, locale);
          returnValue = template.buildTemplate(file);
        } else {
          // nick change allowed.
        }
      }

      if (proceed) {

        user.setNickname(nickname);
        user.setSurename(surename);
        user.setForename(forename);
        user.setEmail(email);

        try {
          userAdmin.editUser(user);
          template.put("info", rb.getString("edit_Confirmed"));

          String file = template.loadFile(start, locale);
          returnValue = template.buildTemplate(file);
        } catch( Exception exc ) {
          template.put("errorcode", "attempt tp change account");
          template.put("errorcontext", "handleEvents: event = " + event);
          template.put("errorstring", "attempt tp change account");
          throw new Exception("logged in user tried to change his/her account. Check database functionality!<br>" + exc.getMessage());
        }
      }
      return returnValue;

/**************************************************************************
LOGOUT
***************************************************************************/

    } else if (event.equals("logout")) {
		System.err.println("logging out... checking acceptance");
      if(userAdmin.acceptUser(user)) {
        user.setSecureIdentity("");
        try {
          userAdmin.editUser(user);
        } catch ( Exception exc ) {
          // couldn't edit user
          // database failure.
        }
        template.put("id","");
        template.put("secureidentity","");
        template.put("info", rb.getString("logout_cu") + user.getNickname() + rb.getString("logout_actions"));

        String file = template.loadFile(login, locale);
        returnValue = template.buildTemplate(file);
        return returnValue;
      }
    }
    // event was not handled
    return null;
  }
}
